@@ -13,7 +13,7 @@ from .lexer import Lexer
 from .parser import Parser
 
 
-def load(file):
+def load(file, **options):
     """
     Read JSON or JSON-ish data structures from `file`, parse them, and build them as Plain Old Data Structures (`dict`, `list`,
     `set` etc).
@@ -22,10 +22,10 @@ def load(file):
 
     Will raise `ParserError` if a syntax error is encountered.
     """
-    return loads(file.read())
+    return loads(file.read(), **options)
 
 
-def loads(text):
+def loads(text, **options):
     """
     Read JSON or JSON-ish data structures from `text`, parse them, and build them as Plain Old Data Structures (`dict`, `list`,
     `set` etc).
@@ -35,13 +35,13 @@ def loads(text):
     Will raise `ParserError` if a syntax error is encountered.
     """
     lexer = Lexer(text)
-    parser = Parser(lexer, PodsBuilder())
+    parser = Parser(lexer, PodsBuilder(**options))
     document = parser.document()
     lexer.end(checked=True)
     return document
 
 
-def translate(builder, input_text):
+def translate(builder, input_text, single_document=False):
     """
     Read the given `input_text` and feed it to the given `Builder`, which should print out a translation of the data to its output
     stream. This is what the command-line tool invokes (see `cli.py`).
@@ -49,10 +49,11 @@ def translate(builder, input_text):
     lexer = Lexer(input_text)
     parser = Parser(lexer, builder)
     try:
-        while not lexer.end():
+        while not (single_document or lexer.end()):
             parser.document()
             builder.output.write(linesep)
-    except Exception:  # anything at all, pylint: disable=broad-except
+        lexer.end(checked=True)
+    except Exception:
         builder.flush()  # finish printing the parsed tokens
         lexer.flush(builder.output)  # then print the unparsed input
         raise
