@@ -292,7 +292,6 @@ ALL_TEST_CASES = [
     Case(
         '(1)',
         expected_parse=ParserError("Position 2: expected `,`, found ')'"),
-        expected_pods=[1],
     ),
     Case(
         '(,)',
@@ -372,18 +371,18 @@ ALL_TEST_CASES = [
         expected_parse=ParserError('Position 1: expected key, found \',"a": 1}\''),
     ),
     Case(
-        '{a: 1}',
+        '{a: a}',
         expected_parse=[
             AST.open_mapping(),
             AST.string('a', 'a'),
             AST.mapping_key(),
-            AST.int('1', 1),
+            AST.identifier('a'),
             AST.mapping_value(),
             AST.close_mapping(),
         ],
-        expected_pods={'a': 1},
-        expected_verb='{a: 1}',
-        expected_json='{"a": 1}',
+        expected_pods={'a': '$$a'},
+        expected_verb='{a: a}',
+        expected_json='{"a": "$$a"}',
     ),
 
     # # Mappings with tuples as keys
@@ -478,6 +477,20 @@ ALL_TEST_CASES = [
         expected_verb='{1, 2}',
         expected_json='{"$set": [1, 2]}',
     ),
+    Case(
+        '{a, b}',
+        expected_parse=[
+            AST.open_set(),
+            AST.identifier('a'),
+            AST.set_element(),
+            AST.identifier('b'),
+            AST.set_element(),
+            AST.close_set(),
+        ],
+        expected_pods={'$$a', '$$b'},
+        expected_verb='{a, b}',
+        expected_json='{"$set": ["$$a", "$$b"]}',
+    ),
 
     # Single-quoted strings
     Case(
@@ -642,7 +655,8 @@ ALL_TEST_CASES = [
         expected_parse=[
             AST.open_function_call('x'),
             AST.int('1', 1),
-            AST.function_argument(),
+            AST.function_call_positional_argument(),
+            AST.function_call_end_positional_arguments(),
             AST.close_function_call(),
         ],
         expected_pods={'$$x': [1]},
@@ -654,7 +668,8 @@ ALL_TEST_CASES = [
         expected_parse=[
             AST.open_function_call('x'),
             AST.int('1', 1),
-            AST.function_argument(),
+            AST.function_call_positional_argument(),
+            AST.function_call_end_positional_arguments(),
             AST.close_function_call(),
         ],
         expected_pods={'$$x': [1]},
@@ -666,9 +681,10 @@ ALL_TEST_CASES = [
         expected_parse=[
             AST.open_function_call('x'),
             AST.int('1', 1),
-            AST.function_argument(),
+            AST.function_call_positional_argument(),
             AST.int('2', 2),
-            AST.function_argument(),
+            AST.function_call_positional_argument(),
+            AST.function_call_end_positional_arguments(),
             AST.close_function_call(),
         ],
         expected_pods={'$$x': [1, 2]},
@@ -676,6 +692,75 @@ ALL_TEST_CASES = [
         expected_json='{"$$x": [1, 2]}',
     ),
 
+    # Function calls with kwargs
+    Case(
+        'x(a=1)',
+        expected_parse=[
+            AST.open_function_call('x'),
+            AST.function_call_end_positional_arguments(),
+            AST.function_call_start_keyword_arguments(),
+            AST.string('a', 'a'),
+            AST.function_call_keyword_argument_key(),
+            AST.int('1', 1),
+            AST.function_call_keyword_argument_value(),
+            AST.function_call_end_keyword_arguments(),
+            AST.close_function_call(),
+        ],
+        expected_pods={'$$x': [], '$kwargs': {'a': 1}},
+        expected_verb='x(a=1)',
+        expected_json='{"$$x": [], "$kwargs": {"a": 1}}',
+    ),
+    Case(
+        'x(a=1,)',
+        expected_parse=[
+            AST.open_function_call('x'),
+            AST.function_call_end_positional_arguments(),
+            AST.function_call_start_keyword_arguments(),
+            AST.string('a', 'a'),
+            AST.function_call_keyword_argument_key(),
+            AST.int('1', 1),
+            AST.function_call_keyword_argument_value(),
+            AST.function_call_end_keyword_arguments(),
+            AST.close_function_call(),
+        ],
+        expected_pods={'$$x': [], '$kwargs': {'a': 1}},
+        expected_verb='x(a=1)',
+        expected_json='{"$$x": [], "$kwargs": {"a": 1}}',
+    ),
+    Case(
+        'x(a=1, 2)',
+        expected_parse=ParserError('Position 7: positional argument follows keyword argument'),
+    ),
+    Case(
+        'x(a=1, b)',
+        expected_parse=ParserError('Position 8: positional argument follows keyword argument'),
+    ),
+    Case(
+        'x(a=1,,)',
+        expected_parse=ParserError('Position 6: positional argument follows keyword argument'),
+    ),
+    Case(
+        'x(a=1, b=2)',
+        expected_parse=[
+            AST.open_function_call('x'),
+            AST.function_call_end_positional_arguments(),
+            AST.function_call_start_keyword_arguments(),
+            AST.string('a', 'a'),
+            AST.function_call_keyword_argument_key(),
+            AST.int('1', 1),
+            AST.function_call_keyword_argument_value(),
+            AST.string('b', 'b'),
+            AST.function_call_keyword_argument_key(),
+            AST.int('2', 2),
+            AST.function_call_keyword_argument_value(),
+            AST.function_call_end_keyword_arguments(),
+            AST.close_function_call(),
+        ],
+        expected_pods={'$$x': [], '$kwargs': {'a': 1, 'b': 2}},
+        expected_verb='x(a=1, b=2)',
+        expected_json='{"$$x": [], "$kwargs": {"a": 1, "b": 2}}',
+    ),
+
     # Known function calls
     Case(
         # MongoDB BSON date objects
@@ -683,7 +768,8 @@ ALL_TEST_CASES = [
         expected_parse=[
             AST.open_function_call('Date'),
             AST.string('"1970-09-12"', '1970-09-12'),
-            AST.function_argument(),
+            AST.function_call_positional_argument(),
+            AST.function_call_end_positional_arguments(),
             AST.close_function_call(),
         ],
         expected_pods={'$date': ['1970-09-12']},
@@ -696,7 +782,8 @@ ALL_TEST_CASES = [
         expected_parse=[
             AST.open_function_call('ObjectId'),
             AST.string('"1234"', '1234'),
-            AST.function_argument(),
+            AST.function_call_positional_argument(),
+            AST.function_call_end_positional_arguments(),
             AST.close_function_call(),
         ],
         expected_pods={'$oid': ['1234']},
@@ -713,7 +800,7 @@ def test_parser(input_text, expected_parse, expected_pods, expected_verb, expect
     mock_builder = AstClass()
     parser = Parser(lexer, mock_builder)
     try:
-        actual_parse = parser.document(include_single_values=True)
+        actual_parse = parser.document()
     except ParserError as actual_error:
         if isinstance(expected_parse, ParserError):
             assert str(actual_error) == str(expected_parse)
@@ -730,7 +817,7 @@ def test_pods(input_text, expected_parse, expected_pods, expected_verb, expected
     builder = PodsBuilder()
     parser = Parser(lexer, builder)
     try:
-        actual_pods = parser.document(include_single_values=True)
+        actual_pods = parser.document()
     except ParserError as actual_error:
         if isinstance(expected_parse, ParserError):
             assert str(actual_error) == str(expected_parse)
@@ -749,7 +836,7 @@ def test_verb(input_text, expected_parse, expected_pods, expected_verb, expected
     builder = VerbatimPrinter(output_buffer, indent=0)
     parser = Parser(lexer, builder)
     try:
-        parser.document(include_single_values=True)
+        parser.document()
         actual_verb = output_buffer.getvalue().rstrip()
     except ParserError as actual_error:
         if isinstance(expected_parse, ParserError):
@@ -769,7 +856,7 @@ def test_json_output(input_text, expected_parse, expected_pods, expected_verb, e
     builder = JsonPrinter(output_buffer, indent=0)
     parser = Parser(lexer, builder)
     try:
-        parser.document(include_single_values=True)
+        parser.document()
         actual_json = output_buffer.getvalue().rstrip()
     except ParserError as actual_error:
         if isinstance(expected_parse, ParserError):
